% File: Cal_endsc.m
% Purpose: Calibration of displaced endstopped neurons
% Parameters:
%   fesc: structure with complex endstopped neuron parameters (see main)
%   fac: These are weights, the c_subc in equation 4
%   ndir: orientations (e.g. 8)
%   plt: plot ranges of curvature detected (logical, 1->yes, 0->no)
% Returns:
%   m, ms and mc are maximum values for displaced endstopped neurons (m)
%       signed endstopped neurons (ms) and corner neurons (mc)
%       These values are useful for normalizing
%   l and h are the range of curvatures for each neuron
%       e.g. l(1)=5, h(1)=12, this neuron is selective for curvatures
%            5 to 12 pixels (sharp curves)
%
% Author: Antonio J. Rodriguez-Sanchez
%
% CVR, Intelligent and Interactive Systems
% Copyright (c) 2012, York University/University of Innsbruck
%
% Permission is granted to copy and distribute this file, for noncommercial
% use, provided (a) this copyright notice is preserved, (b) no attempt
% is made to restrict redistribution of this file, and (c) this file is
% restricted by a compilation copyright.
%
%  All comments concerning this program may be sent to the
%  e-mail address 'Antonio.Rodriguez-Sanchez@uibk.ac.at'
%

function [m ms mc l h] = Cal_endsc(fesc, fac, ndir, plt)

lim = [0 150];
x = repmat(1:600,[600 1]);
y = x';  
caux = zeros(length(fesc),600,600);


% Response to corners
img = zeros(200,200);
img(1:200,100) = 255;
img(100,1:200) = 255;
layerV1 = V1(img, 'DOG', fesc, ndir);
layerV2 = V2_endsc(layerV1, fac, [fesc.size]);
mc = layerV2.corner(1,:,100,100);

% Response to curvatures
i = 1;
for rad=lim(1):5:lim(2)
  img = zeros(600,600);
  a = sqrt((x-300).^2+(y-300).^2)<rad+1 & sqrt((x-300).^2+(y-300).^2)>=rad;
  img(a) = 255;
  img(1:300,:) = 0;
    
  % ------- Complex endstopped --------
  layerV1 = V1(img, 'DOG', fesc, ndir);
  layerV2 = V2_endsc(layerV1, fac, [fesc.size]);
  ends = layerV2.endsc;
  esc(:,i) = ends(1,:,300+rad,300);
  esc2(:,i) = max(max(ends(1,:,:,:),[],3),[],4);    
  c(:,i) = max(max(layerV1.complex_e(1,:,:,:),[],3),[],4);
  esc3(:,i) = layerV2.signn(1,:,300+rad,300);
  
  i = i+1;
end

% Interpolated response to curvatures
% ------- Complex endstopped ---------
test_esc = zeros(3,600,600);
esc(esc<0) = 0;

for i=1:length(fesc)
  esc_sp(i,:) = spline(lim(1):5:lim(2),esc(i,:),lim(1):lim(2));
  [max_e ind] = max(esc_sp(i,:));
  xe = 1:size(esc_sp,2);
  x2 = esc_sp(i,:);
  aux = min(x2(x2(xe)>=0.85*max_e&xe<=ind));
  l(i) = find(x2>=aux, 1 );
  aux = min(x2(x2(xe)>=0.85*max_e&xe>=ind));
  h(i) = find(x2>=aux, 1, 'last' );
  a = sqrt((x-300).^2+(y-300).^2)<ind+1 & sqrt((x-300).^2+(y-300).^2)>ind-1;
  test_esc(i,a) = 255;
end

m = max(esc_sp,[],2);
ms = max(esc3,[],2);

if plt
  figure, plot(c');
  title('Complex neurons')
  xlabel('Radius')
  ylabel('Neuron response');
  figure,
  for i=1:length(fesc)
    subplot(2,2,i); 
    plot(esc_sp(i,:));
    axis([lim(1) lim(2) 0 max(esc_sp(i,:))+0.1]);
    title(['ES complex neuron ', num2str(i)]);
    xlabel('Radius')
    ylabel('Neuron response');
  end
  figure,
  for i=1:length(fesc)
    subplot(2,2,i); 
    plot(esc2(i,:));
    axis([lim(1) lim(2)/5 0 max(esc2(i,:))+0.1]);
    title(['ES complex neuron ', num2str(i)]);
    xlabel('Radius')
    ylabel('Neuron response');
  end
  figure,
  for i=1:length(fesc)
    subplot(2,2,i); 
    plot(esc3(i,:));
%     axis([lim(1) lim(2)/5 0 max(esc3(i,:))+0.1]);
    title(['ES sign ', num2str(i)]);
    xlabel('Radius')
    ylabel('Neuron response');
  end
end

% Response to corners
% for i=1:ndir
%   img = zeros(600,600);
%   img(1:600,300) = 255;
%   for x=1:600, img(round(x*sin(i/ndir)),x) = 255; end
%   layerV1 = V1(img, 'DOG', fesc, ndir);
%   layerV2 = V2_endsc(layerV1, fac, [fesc.size]);
%   mc(i,:) = layerV1.simple_e(i,:,300,300);
% end

