% File: V1.m
% Purpose: V1 layer->simple and complex neurons
% Parameters:
%   - type: 'DOG' or 'Gabor'
%   - img: img to filter
%   - f: vector of filters
%   - ndir: number of directions
% Returns:
%   V1: struct with simple and complex neuron values.
%    - max is the maximum values, angles is at which orientations
%      these are not used but left just in case is needed in the future
%    - simple_e and simple_o are the values of simple neurons for the 
%      even and odd part
%    - complex_e and complex_o are the values of complex neurons
%
% Author: Antonio J. Rodriguez-Sanchez
%
% CVR, Intelligent and Interactive Systems
% Copyright (c) 2012, York University/University of Innsbruck
%
% Permission is granted to copy and distribute this file, for noncommercial
% use, provided (a) this copyright notice is preserved, (b) no attempt
% is made to restrict redistribution of this file, and (c) this file is
% restricted by a compilation copyright.
%
%  All comments concerning this program may be sent to the
%  e-mail address 'Antonio.Rodriguez-Sanchez@uibk.ac.at'
%

function V1 = V1(img, t, f, ndir)

coeff = pi/ndir;
imgfft = fft2(double(img)); 
V1 = struct('max', zeros(size(img,1), size(img,2)), ...
    'angles', zeros(size(img,1), size(img,2)), ...
    'simple_e', zeros(ndir, length(f), size(img,1), size(img,2)), ...
    'simple_o', zeros(ndir, length(f), size(img,1), size(img,2)), ...
    'complex_e', zeros(ndir, length(f), size(img,1), size(img,2)), ...
    'complex_o', zeros(ndir, length(f), size(img,1), size(img,2)));
  
i = 1;
% Gaussian weights
p = -2:2;
w = 1./(sqrt(2*pi)).*exp(-(p).^2./2)*2.5;

for th=0:coeff:pi-coeff % distintas orientaciones
  if(th<=pi/2)   % Esto es solo para los filtros
    theta = abs(th-pi/2);
  else
    theta = 3*pi/2-th;
  end

  for j=1:length(f) % diferentes tama?os de neuronas
    if strcmp(t,'Gabor'); % Filtro: Gabor o DOG
      [fe fo] = ...
        GaborD(f(j).size,f(j).sigma_y,f(j).sigma_x1,theta,f(j).pr,f(j).x0,f(j).y0);
    else
      [fe fo] = ...
        DOG(f(j).size,f(j).sigma_y,f(j).sigma_x1,f(j).sigma_x2,theta,f(j).x0,f(j).y0);
    end

    % neuronas simples
    V1.simple_e(i,j,:,:) = conv2(double(img),fe,'same');        
    V1.simple_o(i,j,:,:) = conv2(double(img),fo,'same');
    
    % neuronas complejas
    se = 2*(f(j).sigma_y/f(j).sigma_x1);
    a = int16((f(j).x0+floor(f(j).size/se))*sin(mod(th+pi/2,pi)));
    b = int16((f(j).y0+floor(f(j).size/se))*cos(mod(th+pi/2,pi)));
    auxe = V1.simple_e(i,j,:,:); auxo = V1.simple_o(i,j,:,:);
    auxe(auxe<0) = 0; auxo(auxo<0) = 0;  
    if(b<0)
      b = abs(b);
      V1.complex_e(i,j,:,:) = auxe(1,1,:,:)*w(3);
      V1.complex_e(i,j,1:size(auxe,3)-a,1:size(auxe,4)-b) = ...
        V1.complex_e(i,j,1:size(auxe,3)-a,1:size(auxe,4)-b) + ...
        auxe(1,1,a+1:size(auxe,3),b+1:size(auxe,4))*w(2);      
      V1.complex_e(i,j,1:size(auxe,3)-2*a,1:size(auxe,4)-2*b) = ...
        V1.complex_e(i,j,1:size(auxe,3)-2*a,1:size(auxe,4)-2*b) + ...
        auxe(1,1,2*a+1:size(auxe,3),2*b+1:size(auxe,4))*w(1);       
      V1.complex_e(i,j,a+1:size(auxe,3),b+1:size(auxe,4)) = ...
        V1.complex_e(i,j,a+1:size(auxe,3),b+1:size(auxe,4)) + ...
        auxe(1,1,1:size(auxe,3)-a,1:size(auxe,4)-b)*w(4);      
      V1.complex_e(i,j,2*a+1:size(auxe,3),2*b+1:size(auxe,4)) = ...
        V1.complex_e(i,j,2*a+1:size(auxe,3),2*b+1:size(auxe,4)) + ...
        auxe(1,1,1:size(auxe,3)-2*a,1:size(auxe,4)-2*b)*w(5);
    else
      V1.complex_e(i,j,:,:) = auxe(1,1,:,:)*w(3);     
      V1.complex_e(i,j,1:size(auxe,3)-a,4*b+1:size(auxe,4)) = ...
        V1.complex_e(i,j,1:size(auxe,3)-a,4*b+1:size(auxe,4)) + ...
        auxe(1,1,a+1:size(auxe,3),3*b+1:size(auxe,4)-b)*w(2);
      V1.complex_e(i,j,1:size(auxe,3)-2*a,4*b+1:size(auxe,4)) = ...
        V1.complex_e(i,j,1:size(auxe,3)-2*a,4*b+1:size(auxe,4)) + ...
        auxe(1,1,2*a+1:size(auxe,3),2*b+1:size(auxe,4)-2*b)*w(1); 
      V1.complex_e(i,j,a+1:size(auxe,3),3*b+1:size(auxe,4)-b) = ...
        V1.complex_e(i,j,a+1:size(auxe,3),3*b+1:size(auxe,4)-b) + ...
        auxe(1,1,1:size(auxe,3)-a,4*b+1:size(auxe,4))*w(4);
      V1.complex_e(i,j,2*a+1:size(auxe,3),2*b+1:size(auxe,4)-2*b) = ...
        V1.complex_e(i,j,2*a+1:size(auxe,3),2*b+1:size(auxe,4)-2*b) + ...
        auxe(1,1,1:size(auxe,3)-2*a,4*b+1:size(auxe,4))*w(5);      
    end      
  end

  i = i+1;  
end

% Seleccionar maximos y angulos correspondientes
[V1.max, V1.angles] = max(V1.simple_e(:,length(f),:,:),[],1);
V1.max = reshape(V1.max,[size(img,1) size(img,2)]);
V1.angles = reshape(V1.angles,[size(img,1) size(img,2)]);
V1.angles = double((V1.angles-1)*coeff);
V1.angles(V1.max<1) = -1;

